@@ -28,7 +28,7 @@ SEXP R_OGR_CAPI_features(SEXP dsn, SEXP layer)
     OGRGeometryH hRing, hRingM;
 
     int navailable_layers; 
-    int i, j, k, km;
+    int i, j, k, km, jcnt;
 /*    int iDriver;*/
     int dim, with_z;
     int np, nr, nm;
@@ -55,14 +55,15 @@ SEXP R_OGR_CAPI_features(SEXP dsn, SEXP layer)
 
     if (j < 0) error("Layer not found");
 
-    PROTECT(ans = NEW_LIST(6)); pc++;
-    PROTECT(ansnames = NEW_CHARACTER(6)); pc++;
+    PROTECT(ans = NEW_LIST(7)); pc++;
+    PROTECT(ansnames = NEW_CHARACTER(7)); pc++;
     SET_STRING_ELT(ansnames, 0, COPY_TO_USER_STRING("dsn"));
     SET_STRING_ELT(ansnames, 1, COPY_TO_USER_STRING("layer"));
     SET_STRING_ELT(ansnames, 2, COPY_TO_USER_STRING("proj4string"));
     SET_STRING_ELT(ansnames, 3, COPY_TO_USER_STRING("geomTypes"));
     SET_STRING_ELT(ansnames, 4, COPY_TO_USER_STRING("crdlist"));
     SET_STRING_ELT(ansnames, 5, COPY_TO_USER_STRING("with_z"));
+    SET_STRING_ELT(ansnames, 6, COPY_TO_USER_STRING("isNULL"));
     setAttrib(ans, R_NamesSymbol, ansnames);
 
     SET_VECTOR_ELT(ans, 0, NEW_CHARACTER(1));
@@ -86,19 +87,24 @@ SEXP R_OGR_CAPI_features(SEXP dsn, SEXP layer)
     SET_VECTOR_ELT(ans, 3, NEW_INTEGER(nf));
     SET_VECTOR_ELT(ans, 4, NEW_LIST(nf));
     SET_VECTOR_ELT(ans, 5, NEW_INTEGER(nf));
+    SET_VECTOR_ELT(ans, 6, NEW_INTEGER(nf));
 
     i=0;
     while( (Ogr_feature = OGR_L_GetNextFeature(Ogr_layer)) != NULL ) {
 	    /* Geometry */
 	Ogr_geometry = OGR_F_GetGeometryRef(Ogr_feature);
 	with_z = 0;
-	if ( Ogr_geometry == NULL ) {
-	    error("NULL geometry found");
-	} else {
+/* TODO fix NULL reading */
+	if ( Ogr_geometry == NULL ||
+          wkbFlatten(OGR_G_GetGeometryType(Ogr_geometry)) == 0) {
+/*	    error("NULL geometry found"); */
+	    INTEGER_POINTER(VECTOR_ELT(ans, 6))[i] = 1;
+ 	} else {
+	    INTEGER_POINTER(VECTOR_ELT(ans, 6))[i] = 0;
 	    dim = OGR_G_GetCoordinateDimension(Ogr_geometry);
 	    if (dim > 2) 
 		with_z = 1;
-	}
+/*	} */
         eType = wkbFlatten(OGR_G_GetGeometryType(Ogr_geometry));
 
 	INTEGER_POINTER(VECTOR_ELT(ans, 3))[i] =  eType;
@@ -128,10 +134,11 @@ SEXP R_OGR_CAPI_features(SEXP dsn, SEXP layer)
 		    ans, 4), i), 0), 2))[0] = OGR_G_GetZ(Ogr_geometry, 0);
 	    }
 	} else if (eType == wkbLineString) {
+	  np = OGR_G_GetPointCount(Ogr_geometry);
+          if (np > 0) {
 	    SET_VECTOR_ELT(VECTOR_ELT(ans, 4), i, NEW_LIST(1));
 	    SET_VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(ans, 4), i), 0, 
 		NEW_LIST(2));
-	    np = OGR_G_GetPointCount(Ogr_geometry);
 
 	    SET_VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(ans, 4), i), 0), 0,
 		NEW_NUMERIC(np));
@@ -143,14 +150,19 @@ SEXP R_OGR_CAPI_features(SEXP dsn, SEXP layer)
 	        NUMERIC_POINTER(VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(
 		    ans, 4), i), 0), 1))[j] = OGR_G_GetY(Ogr_geometry, j);
 	    }
+          } else INTEGER_POINTER(VECTOR_ELT(ans, 6))[i] = 1;
 	} else if (eType == wkbPolygon) {
-	    nr = OGR_G_GetGeometryCount(Ogr_geometry);
+	  nr = OGR_G_GetGeometryCount(Ogr_geometry);
+          if (nr > 0) {
 	    SET_VECTOR_ELT(VECTOR_ELT(ans, 4), i, NEW_LIST(nr));
+            jcnt = 0;
 	    for (j=0; j < nr; j++) {
+	      hRing = OGR_G_GetGeometryRef(Ogr_geometry, j);
+	      np = OGR_G_GetPointCount(hRing);
+              if (np > 0) {
+                jcnt++;
 	        SET_VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(ans, 4), i), j, 
 		    NEW_LIST(2));
-	        hRing = OGR_G_GetGeometryRef(Ogr_geometry, j);
-	        np = OGR_G_GetPointCount(hRing);
 	        SET_VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(ans, 4), i), 
 		    j), 0, NEW_NUMERIC(np));
 	        SET_VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(ans, 4), i), 
@@ -162,15 +174,22 @@ SEXP R_OGR_CAPI_features(SEXP dsn, SEXP layer)
 	            NUMERIC_POINTER(VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(
 		        ans, 4), i), j), 1))[k] = OGR_G_GetY(hRing, k);
 	        }
+              } 
 	    }
+            if (jcnt == 0) INTEGER_POINTER(VECTOR_ELT(ans, 6))[i] = 1;
+          } else INTEGER_POINTER(VECTOR_ELT(ans, 6))[i] = 1;
 	} else if (eType == wkbMultiLineString) {
-	    nm = OGR_G_GetGeometryCount(Ogr_geometry);
+	  nm = OGR_G_GetGeometryCount(Ogr_geometry);
+          if (nm > 0) {
 	    SET_VECTOR_ELT(VECTOR_ELT(ans, 4), i, NEW_LIST(nm));
+            jcnt = 0;
 	    for(j = 0; j < nm; j++) {
+	      hRingM = OGR_G_GetGeometryRef(Ogr_geometry, j);
+	      np = OGR_G_GetPointCount(hRingM);
+              if (np > 0) {
+                jcnt++;
 	        SET_VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(ans, 4), i), j, 
 		    NEW_LIST(2));
-		hRingM = OGR_G_GetGeometryRef(Ogr_geometry, j);
-	        np = OGR_G_GetPointCount(hRingM);
 	        SET_VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(ans, 4), i), 
 		    j), 0, NEW_NUMERIC(np));
 	        SET_VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(ans, 4), i), 
@@ -181,9 +200,13 @@ SEXP R_OGR_CAPI_features(SEXP dsn, SEXP layer)
 	            NUMERIC_POINTER(VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(
 		        ans, 4), i), j), 1))[k] = OGR_G_GetY(hRingM, k);
 	        }
+              }
 	    }
+            if (jcnt == 0) INTEGER_POINTER(VECTOR_ELT(ans, 6))[i] = 1;
+          } else INTEGER_POINTER(VECTOR_ELT(ans, 6))[i] = 1;
 	} else if (eType == wkbMultiPolygon) {
-	    nm = OGR_G_GetGeometryCount(Ogr_geometry);
+	  nm = OGR_G_GetGeometryCount(Ogr_geometry);
+          if (nm > 0) {
 	    for(j = 0, mp_count = 0; j < nm; j++) {
 		hRingM = OGR_G_GetGeometryRef(Ogr_geometry, j);
 	        nr = OGR_G_GetGeometryCount(hRingM);
@@ -191,33 +214,43 @@ SEXP R_OGR_CAPI_features(SEXP dsn, SEXP layer)
 	    }
 	    SET_VECTOR_ELT(VECTOR_ELT(ans, 4), i, NEW_LIST(mp_count));
 	    mp_count = 0;
+            jcnt = 0;
 	    for(j = 0; j < nm; j++) {
 		hRingM = OGR_G_GetGeometryRef(Ogr_geometry, j);
 	        nr = OGR_G_GetGeometryCount(hRingM);
-	        for (k=0; k < nr; k++) {
-	            SET_VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(ans, 4), i), mp_count, 
-		        NEW_LIST(2));
+                if (nr > 0) {
+	          for (k=0; k < nr; k++) {
 	            hRing = OGR_G_GetGeometryRef(hRingM, k);
 	            np = OGR_G_GetPointCount(hRing);
-	            SET_VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(ans, 4), 
+                    if (np > 0) {
+                      jcnt++;
+	              SET_VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(ans, 4), i),
+                        mp_count, NEW_LIST(2));
+	              SET_VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(ans, 4), 
 			i), mp_count), 0, NEW_NUMERIC(np));
-	            SET_VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(ans, 4), 
+	              SET_VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(ans, 4), 
 			i), mp_count), 1, NEW_NUMERIC(np));
-                    for(km = 0; km < np; km++) {
-	                NUMERIC_POINTER(VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(
+                      for(km = 0; km < np; km++) {
+	                  NUMERIC_POINTER(VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(
 			    VECTOR_ELT(ans, 4), i), mp_count), 0))[km] = 
 			    OGR_G_GetX(hRing, km);
-	                NUMERIC_POINTER(VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(
+	                  NUMERIC_POINTER(VECTOR_ELT(VECTOR_ELT(VECTOR_ELT(
 			    VECTOR_ELT(ans, 4), i), mp_count), 1))[km] = 
 			    OGR_G_GetY(hRing, km);
-	            }
-		    mp_count++;
+	              }
+		      mp_count++;
+                    }
+                  }
 	        }
-	    }
+	    } 
+            if (jcnt == 0) INTEGER_POINTER(VECTOR_ELT(ans, 6))[i] = 1;
+          } else INTEGER_POINTER(VECTOR_ELT(ans, 6))[i] = 1;
 	} else warning("eType not chosen");
 	OGR_F_Destroy(Ogr_feature);
-	i++;
+      }
+      i++;
     }
+    OGR_DS_Destroy(Ogr_ds);
     UNPROTECT(pc);
 
     return(ans);
@@ -236,7 +269,7 @@ SEXP R_OGR_types(SEXP dsn, SEXP layer)
     int navailable_layers; 
     int i, j;
 /*    int iDriver;*/
-    int dim, with_z;
+    int dim, with_z, isNULL;
 /*    char *pszProj4 = NULL;*/
 
     int pc=0;
@@ -260,13 +293,14 @@ SEXP R_OGR_types(SEXP dsn, SEXP layer)
 
     if (j < 0) error("Layer not found");
 
-    PROTECT(ans = NEW_LIST(5)); pc++;
-    PROTECT(ansnames = NEW_CHARACTER(5)); pc++;
+    PROTECT(ans = NEW_LIST(6)); pc++;
+    PROTECT(ansnames = NEW_CHARACTER(6)); pc++;
     SET_STRING_ELT(ansnames, 0, COPY_TO_USER_STRING("dsn"));
     SET_STRING_ELT(ansnames, 1, COPY_TO_USER_STRING("layer"));
     SET_STRING_ELT(ansnames, 2, COPY_TO_USER_STRING("proj4string"));
     SET_STRING_ELT(ansnames, 3, COPY_TO_USER_STRING("geomTypes"));
-    SET_STRING_ELT(ansnames, 5, COPY_TO_USER_STRING("with_z"));
+    SET_STRING_ELT(ansnames, 4, COPY_TO_USER_STRING("with_z"));
+    SET_STRING_ELT(ansnames, 5, COPY_TO_USER_STRING("isNULL"));
     setAttrib(ans, R_NamesSymbol, ansnames);
 
     SET_VECTOR_ELT(ans, 0, NEW_CHARACTER(1));
@@ -289,27 +323,37 @@ SEXP R_OGR_types(SEXP dsn, SEXP layer)
 
     SET_VECTOR_ELT(ans, 3, NEW_INTEGER(nf));
     SET_VECTOR_ELT(ans, 4, NEW_INTEGER(nf));
+    SET_VECTOR_ELT(ans, 5, NEW_INTEGER(nf));
 
     i=0;
     while( (Ogr_feature = OGR_L_GetNextFeature(Ogr_layer)) != NULL ) {
 	    /* Geometry */
 	Ogr_geometry = OGR_F_GetGeometryRef(Ogr_feature);
 	with_z = 0;
-	if ( Ogr_geometry == NULL ) {
-	    error("NULL geometry found");
-	} else {
+        isNULL = 0;
+        eType = wkbUnknown;
+	if (Ogr_geometry == NULL) {
+/*	    error("NULL geometry found"); */
+	    isNULL = 1;
+ 	} else {
 	    dim = OGR_G_GetCoordinateDimension(Ogr_geometry);
-	    if (dim > 2) 
-		with_z = 1;
-	}
-        eType = wkbFlatten(OGR_G_GetGeometryType(Ogr_geometry));
+	    if (dim > 2) with_z = 1;
+            eType = wkbFlatten(OGR_G_GetGeometryType(Ogr_geometry));
+        }
+/* ogrInfo("/home/rsb/tmp/bigshape", "parcel_mast_20050802") */
+/* mast <- readOGR("/home/rsb/tmp/bigshape", "parcel_mast_20050802") */
 
-	INTEGER_POINTER(VECTOR_ELT(ans, 3))[i] =  eType;
-	INTEGER_POINTER(VECTOR_ELT(ans, 4))[i] =  with_z;
+	if (eType == wkbUnknown || eType == wkbNone) isNULL = 1;
+
+	INTEGER_POINTER(VECTOR_ELT(ans, 3))[i] = eType;
+	INTEGER_POINTER(VECTOR_ELT(ans, 4))[i] = with_z;
+	INTEGER_POINTER(VECTOR_ELT(ans, 5))[i] = isNULL;
 
 	OGR_F_Destroy(Ogr_feature);
 	i++;
+      
     }
+    OGR_DS_Destroy(Ogr_ds);
     UNPROTECT(pc);
 
     return(ans);
